/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.view;

import java.io.IOException;

import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import kieker.analysis.AnalysisController;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.beans.application.ProjectsBean;
import kieker.webgui.common.IProjectManagerFacade;
import kieker.webgui.common.ProjectManagerFacade;
import kieker.webgui.common.exception.AnalysisStateException;
import kieker.webgui.common.exception.ProjectLoadException;
import kieker.webgui.common.exception.ProjectNotExistingException;

/**
 * /**
 * The {@link CurrentControllerBean} contains the necessary data behind an instance of the analysis controller. The class is a JSF managed bean with view scope to
 * make sure that one user (even in one session) can open multiple projects at a time without causing any problems.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@ViewScoped
public class CurrentControllerBean {

	private static final Log LOG = LogFactory.getLog(CurrentControllerBean.class);
	private final IProjectManagerFacade projectManagerFacade = ProjectManagerFacade.getInstance();
	private String projectName;
	private MIProject project;
	@ManagedProperty(value = "#{projectsBean}")
	private ProjectsBean projectsBean;

	/**
	 * Creates a new instance of this class. <b>Do not use this constructor. This bean is JSF managed.</b>
	 */
	public CurrentControllerBean() {
		// No code necessary
	}

	/**
	 * The getter for the {@link #projectsBean}-property.
	 * 
	 * @return {@link #projectsBean}
	 */
	public ProjectsBean getProjectsBean() {
		return this.projectsBean;
	}

	/**
	 * The setter for the {@link #projectsBean}-property.
	 * 
	 * @param projectsBean
	 *            The new value for {@link #projectsBean}.
	 */
	public void setProjectsBean(final ProjectsBean projectsBean) {
		this.projectsBean = projectsBean;
	}

	/**
	 * This method sets the project stored within this bean and returns the new page for the navigation.
	 * 
	 * @param newName
	 *            The name of the project.
	 */
	public void setProjectName(final String newName) {
		synchronized (this) {
			// Remember the given parameters
			this.projectName = newName;
		}
	}

	/**
	 * This method initializes the bean by using the current project name to load the project. <b>Do not call this method manually. It will only be accessed by
	 * JSF.</b>
	 */
	public void initalize() {
		synchronized (this) {
			try {
				// Make sure that the initialization will only be done for the init request.
				if (!FacesContext.getCurrentInstance().isPostback()) {
					// Remember the given parameters
					this.project = this.projectsBean.openProject(this.projectName);
				}
			} catch (final ProjectLoadException ex) {
				CurrentControllerBean.LOG.error("An error occured while loading the project.", ex);
				CurrentControllerBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.error("An error occured while loading the project.", ex);
				CurrentControllerBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
			}
		}
	}

	/**
	 * This method delivers the project stored in this bean.
	 * 
	 * @return The project for this user.
	 */
	public MIProject getProject() {
		synchronized (this) {
			return this.project;
		}
	}

	/**
	 * This method delivers the project name stored in this bean.
	 * 
	 * @return The project name for this user.
	 */
	public String getProjectName() {
		synchronized (this) {
			return this.projectName;
		}
	}

	/**
	 * This method starts the current analysis and informs the user about a fail.
	 */
	public void startAnalysis() {
		try {
			synchronized (this) {
				this.projectManagerFacade.startAnalysis(this.projectName);
			}
		} catch (final AnalysisStateException ex) {
			CurrentControllerBean.LOG.info("The analysis has already been started.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "The analysis has already been started.");
		} catch (final ProjectNotExistingException ex) {
			CurrentControllerBean.LOG.info("The project does not exist.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "The project does not exist.");
		}
	}

	/**
	 * This method stops the current analysis and informs the user about a fail.
	 */
	public void stopAnalysis() {
		try {
			synchronized (this) {
				this.projectManagerFacade.stopAnalysis(this.projectName);
			}
		} catch (final AnalysisStateException ex) {
			CurrentControllerBean.LOG.info("The analysis has not been started yet.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "The analysis has not been started yet.");
		} catch (final ProjectNotExistingException ex) {
			CurrentControllerBean.LOG.info("The project does not exist.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "The project does not exist.");
		}
	}

	/**
	 * This method initializes the current analysis and informs the user about a fail.
	 */
	public void instantiateAnalysis() {
		try {
			synchronized (this) {
				this.projectManagerFacade.initializeAnalysis(this.projectName, this.projectManagerFacade.getClassLoader(this.projectName)); // NOPMD (ClassLoader)
			}
		} catch (final AnalysisStateException ex) {
			CurrentControllerBean.LOG.error("The analysis has already been instantiated.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_ERROR, "The analysis has already been instantiated.");
		} catch (final ProjectNotExistingException ex) {
			CurrentControllerBean.LOG.info("The project does not exist.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "The project does not exist.");
		} catch (final IOException ex) {
			CurrentControllerBean.LOG.info("An error occured during the initialization.", ex);
			CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "An error occured during the initialization.");
		}
	}

	/**
	 * This method cleans the current analysis instance.
	 */
	public void cleanAnalysis() {
		synchronized (this) {
			try {
				this.projectManagerFacade.cleanAnalysis(this.projectName);
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				CurrentControllerBean.showMessage(FacesMessage.SEVERITY_WARN, "The project does not exist.");
			} catch (final AnalysisStateException ex) {
				CurrentControllerBean.LOG.error("The analysis has not been instantiated yet.", ex);
				CurrentControllerBean.showMessage(FacesMessage.SEVERITY_ERROR, "The analysis has not been instantiated yet.");
			}
		}
	}

	/**
	 * Checks whether the analysis is currently running.
	 * 
	 * @return true if and only if the analysis is running.
	 */
	public boolean isAnalysisRunning() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.RUNNING;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * Checks whether the analysis is currently in the ready state.
	 * 
	 * @return true if and only if the analysis is ready to be started.
	 */
	public boolean isAnalysisReady() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.READY;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * Checks whether the analysis is not available.
	 * 
	 * @return true if and only if the analysis is <b>not</b> available.
	 */
	public boolean isAnalysisNotAvailable() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == null;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return true;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return true;
			}
		}
	}

	/**
	 * Checks whether the analysis is currently terminated.
	 * 
	 * @return true if and only if the analysis has been terminated.
	 */
	public boolean isAnalysisTerminated() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.TERMINATED;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * Checks whether the analysis is currently in the failed state.
	 * 
	 * @return true if and only if the analysis has failed.
	 */
	public boolean isAnalysisFailed() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.FAILED;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * This method shows the current user a message by using the growl-component of PrimeFaces.
	 * 
	 * @param severity
	 *            The severity of the message.
	 * @param msg
	 *            The message itself.
	 */
	private static void showMessage(final Severity severity, final String msg) {
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(severity, "", msg));
	}
}
